/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "asset.h"
#include <QtCore/QStringList>
#include <QFile>
#include <QDir>

REGISTER_OBJECTTYPE(GluonEngine, Asset)

using namespace GluonEngine;

class GluonEngine::AssetPrivate
{
public:
    QUrl file;
};

Asset::Asset(QObject *parent)
    : GluonObject(parent)
{
    d = new AssetPrivate;
}

Asset::~Asset()
{
    delete d;
}

GluonCore::GluonObject* Asset::instantiate()
{
    return new Asset(this);
}

void Asset::setName(const QString& newName)
{
    QString oldName(name());

    GluonCore::GluonObject::setName(newName);
    
    // Rename the underlying file, if one exists...
    if(QDir::current().exists(d->file.toLocalFile()) && !d->file.isEmpty())
    {
        QUrl newFile(QString("Assets/%1.%2").arg(fullyQualifiedName()).arg(QFileInfo(d->file.toLocalFile()).completeSuffix()));
        if(QDir::current().rename(d->file.toLocalFile(), newFile.toLocalFile()))
        {
            setFile(newFile);
        }
    }
}

void Asset::setFile(const QUrl &newFile)
{
    d->file = newFile;
    emit dataChanged();
}

QUrl Asset::file() const
{
    return d->file;
}

QString
Asset::childrenToGDL(int indentLevel) const
{
    Q_UNUSED(indentLevel)
    // We do not recurse here - this allows the assets to handle their own
    // children
    return QString();
}

#include "asset.moc"
